#include "cpcrslib.h"
#include "morelib.h"
#include "lib/starkos/starkos.h"

#ifdef CASSETTE
#define MAIN_MENU_Y 8
#define PAUSE_MENU_Y 8
#define STAT_MENU_Y 12
#endif

#define MAIN_MENU_X 1
#define PAUSE_MENU_X 8
#define STAT_MENU_X 8

#ifdef DISC
#define MAIN_MENU_Y 10
#define PAUSE_MENU_Y 11
#define STAT_MENU_Y 12
#endif

#define grid_width 8
#define grid_height 8
#define grid_size (grid_width*grid_height)
#define tile_width 12
#define tile_height 23
#define tile_width_bytes (tile_width>>1)
#define tile_size ((tile_width>>1)*tile_height)
#define playfield_width_bytes (96/2)
#define background_width_bytes (96/2)
#define playfield_height_lines 184
#define background_org_x 1
#define background_org_y 8

#define playfield_org_x 1
#define playfield_org_y 8
#define font_height 8

enum
{
	SCR_FRONTEND = 0,
	SCR_GAME
};

enum
{
	GAMEMODE_NORMAL = 0,
	GAMEMODE_TIMER5,
	GAMEMODE_TIMER10,
	GAMEMODE_TIMER15,
	GAMEMODE_HIDDEN,
	GAMEMODE_MAX,
	GAMEMODE_TIMER
};

enum
{
	MODE_PLAYER_MOVE = 0,		/* update decision for move */
	MODE_TAKE_NUMBER,			/* take a number */
	MODE_GRID_DONE,				/* grid is done */
	MODE_MOVE_CURSOR,			/* move cursor */
	MODE_DELAY,					/* perform a delay */
	MODE_BACK_TO_FRONT_END,		/* go back to front end */
	MODE_STATS,					/* show stats */
	MODE_PAUSE,
	MODE_START,
/*	MODE_GRID_SHOW_WINNER,*/		/* icon to show winner and sound */
	MODE_GRID_PLAYER_WIN,		/* reveal picture fully and go to next picture */
	MODE_GRID_DONE_DONE,		/* delay before going to stats */
	MODE_GRID_REMOVE_TILES		/* remove any non-zero tiles */
};	

enum
{
	DIFF_EASY = 0,
	DIFF_MEDIUM,
	DIFF_HARD, 
	DIFF_MAX
};

enum
{
	START_PLAYER1 = 0,
	START_PLAYER2,
	START_RANDOM,
	START_LASTWINNER,
	START_MAX
};

enum
{
	PLAYER_HUMAN = 0,
	PLAYER_CPU
};
#ifdef DISC
extern unsigned char Logo[];
#endif

extern unsigned char Tiles[];

extern unsigned char main_music[];
//#ifdef DISC
extern unsigned char select_sfx[];
extern unsigned char move_sfx[];
//#endif

#define WIN_LOSE_TILE_WIDTH_BYTES 5
#define WIN_LOSE_TILE_HEIGHT 19
#define WIN_LOSE_TILE_SIZE (WIN_LOSE_TILE_WIDTH_BYTES*WIN_LOSE_TILE_HEIGHT)

#ifdef DISC
extern unsigned char game_music[];
extern unsigned char win_music[];
extern unsigned char lost_music[];


extern unsigned char LargeScoreGfx[];
#define LARGE_SCORE_TILE_WIDTH_BYTES 2
#define LARGE_SCORE_TILE_HEIGHT 16
#define LARGE_SCORE_TILE_SIZE (LARGE_SCORE_TILE_WIDTH_BYTES*LARGE_SCORE_TILE_HEIGHT)


extern unsigned char WinLoseGfx[];
#define WIN_HEART_GFX (WinLoseGfx+(WIN_LOSE_TILE_SIZE*0))
#define WIN_HEART_GFX2 (WinLoseGfx+(WIN_LOSE_TILE_SIZE*1))
#define WIN_SKULL_GFX (WinLoseGfx+(WIN_LOSE_TILE_SIZE*2))
#define WIN_SKULL_GFX2 (WinLoseGfx+(WIN_LOSE_TILE_SIZE*3))

static unsigned char *WinGfx[2];
static unsigned char *LoseGfx[2];

//static unsigned char WinnerAnimRepeatCount;

extern unsigned char StopGoGfx[];
#define STOP_GO_GFX_HEIGHT 19
#define STOP_GO_GFX_WIDTH_BYTES 5
#define STOP_GO_TILE_SIZE (STOP_GO_GFX_HEIGHT*STOP_GO_GFX_WIDTH_BYTES)
#define GO_GFX (StopGoGfx+(STOP_GO_TILE_SIZE*0))
#define STOP_GFX (StopGoGfx+(STOP_GO_TILE_SIZE*1))

#define stopgox1 63
#define stopgoy1 124
#define stopgox2 63
#define stopgoy2 170

#endif
static unsigned short WinnerAnimFrame;

static unsigned char RandomSeed;
extern unsigned char BackgroundBuffer[];
static unsigned short WinnerId;
static unsigned char *WinLoseScrAddr;
static signed short PlayerScores[2];
/*static signed char LastTook[2];*/
static unsigned short GamesWon[2];
static signed short PlayGrid[grid_size];
static unsigned short CursorX;
static unsigned short OldCursorX;
static unsigned short CursorY;
static unsigned short OldCursorY;
static unsigned short DestCursorX;
static unsigned short DestCursorY;
static unsigned short draw_blank = 0;
static  signed short Scores[63] = {
	-10,-10,-10, 
	-9,-9,-9,
	-8,-8,-8, 
	-7,-7,-7,
	-6,-6,-6, 
	-5,-5,-5,
	-4,-4,-4, 
	-3,-3,-3,
	-2,-2,-2,
	-1,-1,-1,
	1,1,1,
	2,2,2,
	3,3,3,
	4,4,4,
	5,5,5,
	6,6,6,
	7,7,7,
	8,8,8,
	9,9,9,
	10,10,10,
	11,11,11
};


enum
{
	OP_DIFF = 0,
	OP_GAMEMODE,
	OP_VS,
	OP_STARTMODE,
	OP_SOUND,
	OP_PLAY,
	OP_MAX
};

enum
{
	SOUND_MUSIC = 0,
	SOUND_SFX,
	SOUND_MUTE,
	SOUND_MAX
};

enum
{
	STAT_OP_CONTINUE = 0,
	STAT_OP_QUIT
};

enum
{
	PAUSE_OP_CONTINUE = 0,
	PAUSE_OP_QUIT
};

enum
{
	VS_PLAYERVSCPU = 0,
	VS_CPUVSPLAYER,
	VS_PLAYERVSPLAYER,
	VS_CPUVSCPU,
	VS_MAX
};


//static unsigned char *current_music;
static unsigned short PlayerType[2];
static unsigned short PlayerId;
static unsigned short StartPlayer;
static unsigned short NextMode;
static unsigned short DelayNextMode;
static unsigned short WinnerMode;
static unsigned short showline;
static unsigned short Delay;
static unsigned short Diff = DIFF_EASY;
static unsigned short CurrentDiff;
static unsigned short RoundCount;
static unsigned short GameMode = GAMEMODE_NORMAL;
static unsigned short StartMode = START_PLAYER1;
static unsigned short SoundMode = SOUND_MUSIC;
static unsigned short VS = VS_PLAYERVSCPU;
static unsigned short scr = SCR_FRONTEND;
static unsigned short TimerReset;
static unsigned short Timer;
static unsigned short CurrentMode;
static unsigned short VsyncMark = 0;
static unsigned short ShowWinner = 0;
static unsigned short WinnerDelay = 0;

#ifdef CASSETTE
static unsigned short NumBackgrounds = 5;

extern unsigned char Background1[];
extern unsigned char Background2[];
extern unsigned char Background3[];
extern unsigned char Background4[];
extern unsigned char Background5[];

static unsigned char *Backgrounds[5] = 
{
	Background1,
	Background2,
	Background3,
	Background4,
	Background5
};
#endif
#ifdef DISC
//extern unsigned char Loading[];

static unsigned short NumBackgrounds = 10;
static unsigned char Filename[] = "BANGEL69.BK0";
static unsigned short LoadedBackgroundIndex = 255;
#endif
static unsigned short BackgroundIndex;

#ifdef CASSETTE
void copy_background()
{
	/* decrunch stored buffer into background buffer */
//	cpc_Uncrunch(Backgrounds[BackgroundIndex], BackgroundBuffer);
cpc_UnExo(Backgrounds[BackgroundIndex], BackgroundBuffer);
}
#endif

static unsigned char *tilescraddr;

void grid_to_scr(unsigned short x, unsigned short y)
{
	unsigned short tilescrx = (x*tile_width_bytes)+playfield_org_x;
	unsigned short tilescry = (y*tile_height)+playfield_org_y;	
	tilescraddr = cpc_GetScrAddress(tilescrx,tilescry);
}

void draw_background(unsigned short x,unsigned short y)
{
	cpc_CopySpriteRectangleToScreen((unsigned char *)tilescraddr,
				tile_height,
				tile_width_bytes, 
				BackgroundBuffer,
				x*tile_width_bytes,
				(y*tile_height),
				playfield_width_bytes);
}

void draw_trans_tile(unsigned char *tileaddr)
{
	cpc_PutSpTr(tileaddr, tile_height, tile_width_bytes, tilescraddr);
}

#define GRID_TILE_ADDR (Tiles+(10*tile_size))

signed short GetTile(unsigned short x, unsigned short y)
{
	unsigned short offset = (y*grid_width)+x;
	return PlayGrid[offset];							
}

signed short RemoveTile(unsigned short x, unsigned short y)
{
	unsigned short offset = (y*grid_width)+x;
	signed short Tile = PlayGrid[offset];
	PlayGrid[offset] = 0;
	return Tile;
}


unsigned char *get_tile_addr(signed short Tile)
{
	unsigned short TileGfx;
	
	/* number showing? */
	if (draw_blank)
	{
		TileGfx = (Tile<0) ? 23 : 22;
		
		/* no numbers shown */
//		if (Tile<0)
//		{
//			TileGfx=23;
//		}
//		else
//		{
//			TileGfx=22;
//		}		
	}
	else
	{
		TileGfx = 10+Tile;
	}
	
	/* draw non-transparent tile */
	return Tiles+((unsigned short)tile_size*(unsigned short)TileGfx);	
}

void draw_tile(unsigned char *scraddr, signed short Tile)
{
	unsigned char *tileaddr = get_tile_addr(Tile);
	cpc_PutSp(tileaddr, tile_height, tile_width_bytes, scraddr);
}

void draw_grid_square(unsigned short x, unsigned short y)
{
	signed short Tile = GetTile(x,y);
	
	grid_to_scr(x,y);

	/* picture showing? */
	if (Tile)
	{
		draw_tile(tilescraddr, Tile);
	}
	else
	{
		/* draw background */
		draw_background(x,y);
		
	}

	/* now draw transparent grid tile over it */
	draw_trans_tile(GRID_TILE_ADDR);
}

#define CURSOR_TILE_ADDR (Tiles+(24*tile_size))

void draw_cursor()
{
	if ((OldCursorX!=CursorX) || (OldCursorY!=CursorY))
	{
		/* erase old tile */
		draw_grid_square(OldCursorX, OldCursorY);
	}

	/* draw new tile */
	draw_grid_square(CursorX, CursorY);
	
	/* draw cursor */
	draw_trans_tile(CURSOR_TILE_ADDR);
}

static signed short *gridptr;
static unsigned short r;

/* check row or column is empty */
unsigned short checkEmpty(unsigned short offset, unsigned short increment, unsigned short max)
{
	r = 0;
	gridptr = PlayGrid+offset;
	while (r<max)
	{
		signed short val = gridptr[0];
		if (val!=0)
			return val;
			
		gridptr+=increment;
		r++;
	}
	
	return 0;
}

#if 0
/* find best */
unsigned short findBestOrWorst(unsigned short offset, unsigned short increment, char rmax, unsigned short worst )
{
	unsigned short  r;
	signed short *gridptr = PlayGrid+offset;
	signed short val_best;
	unsigned short val_r;
	
	/* really small number */
	if (worst)
	{
		val_best = 99;
	}
	else
	{
		val_best = -99;
	}
	val_r = -1;

	for (r=0; r<rmax; r++)
	{
		/* value in grid */
		signed short val = gridptr[0];
		
		if (val!=0)
		{
			unsigned short store = 0;
			if (worst)
			{
				if (val<val_best)
				{
					store = 1;
				}
			}
			else
			{
				if (val>val_best)
				{
					store = 1;
				}
			}
			
			/* find the highest value */
			if (store)
			{
				val_best = val;
				val_r = r;
			}
		}
		gridptr+=increment;
	}	
	
	return val_r;
}
#endif

#define MAX_MOVES 4
/* x coordinate stored */
static unsigned short StoredCoords[MAX_MOVES*2];
/* num stored */
static unsigned short  NumStored;
/* player scores stored */
static signed short StoredScores[MAX_MOVES+2];

static unsigned short  BestCoords[2];
//static signed short BstDst;
static signed int BstDst;

static unsigned short  BestCoordsSet;
static unsigned short CurrentAIPlayer;
static unsigned short StoredAIPlayer;

void InitAIMove()
{
	BestCoordsSet = NumStored = 0;
	StoredAIPlayer = CurrentAIPlayer = PlayerId;

	/* store initial scores */
	if (PlayerId==0)
	{
		StoredScores[0] = PlayerScores[0];
		StoredScores[1] = PlayerScores[1];
	}
	else
	{
		StoredScores[0] = PlayerScores[1];
		StoredScores[1] = PlayerScores[0];
	}
}

/* return 1 if this square is safe to take e.g. not taken already */
unsigned short AISquareNotChosen(unsigned short x, unsigned short y)
{
	unsigned short i;
	for (i=0; i<NumStored<<1; i+=2)
	{
		/* we have taken this square already */
		if ((StoredCoords[i]==x) && (StoredCoords[i+1]==y))
			return 0;
	}
	return 1;
}

void SwapAIPlayer()
{
/* swap player */
					CurrentAIPlayer = CurrentAIPlayer^1;
}

void DoAIMove(unsigned short xstart, unsigned short ystart, unsigned short Depth)
{
	unsigned short checkmove = 1;
	if (Depth!=0)
	{
		unsigned short x,y;
		unsigned short xinc, yinc;
		/* based on current player, choose where to scan grid */
		if (CurrentAIPlayer==0)
		{
			xinc=1;
			yinc=0;
			x = 0;
			y = ystart;
		}
		else
		{
			xinc=0;
			yinc=1;
			x = xstart;
			y = 0;
		}
		/* scan whole column or row */
		while ((x<grid_width) && (y<grid_height))
		{
			/* get tile */
			signed short Tile = GetTile(x,y);

			/* tile not taken */
			if (Tile!=0)
			{
				/* tile not chosen */
				if (AISquareNotChosen(x,y))
				{
					/* get score offset in table */
					unsigned short ScoreOffset = NumStored;
					/* update score */
					/* get current score */
					signed short CurrentScore = StoredScores[ScoreOffset];
					/* calc new */
					signed short NewScore = CurrentScore + (signed short)Tile;
					unsigned short CoordOffset;
					/* store new */
					StoredScores[ScoreOffset+2] = NewScore;

					/* not reached depth yet */
					/* store the coords chosen */
					CoordOffset = NumStored<<1;
					StoredCoords[CoordOffset] = x;
					StoredCoords[CoordOffset+1] = y;

					NumStored++;

					/* swap player */
					SwapAIPlayer();

					/* do move for this player */
					DoAIMove(x,y,Depth-1);

					SwapAIPlayer();

/*					StoredScores[ScoreOffset+2] = 0; */

					NumStored--;

					checkmove = 0;
				}
			}
			/* increment x and y to move accross or down grid */
			x += xinc;
			y += yinc;
		}
	}

	if (checkmove)
	{
		/* reached depth */
		unsigned short ScoreOffsetStartPlayer;
		unsigned short ScoreOffsetNextPlayer;
		signed short ScoreStartPlayer;
		signed short ScoreNextPlayer;
		signed short newdist;

		// copy previous score for other player
		if ((NumStored & 1)!=0)
		{
			StoredScores[NumStored+2] = StoredScores[NumStored];
		}

		/* now check if we would win */
		ScoreOffsetStartPlayer = NumStored;
		ScoreStartPlayer = StoredScores[ScoreOffsetStartPlayer];

	//	CurrentAIPlayer = CurrentAIPlayer^1;

		ScoreOffsetNextPlayer = NumStored+1;
		ScoreNextPlayer = StoredScores[ScoreOffsetNextPlayer];

	//	CurrentAIPlayer = CurrentAIPlayer^1;


		// if my score is higher choose it
		// if my score is lower don't choose it unless it makes me closer
		newdist = ScoreStartPlayer - ScoreNextPlayer;

		if (BestCoordsSet==0)
		{
			unsigned short x = StoredCoords[0];
			unsigned short y = StoredCoords[1];
			BestCoords[0] = x;
			BestCoords[1] = y;
			BestCoordsSet = 1;

			BstDst = newdist;
		}
		else
		{
			if (newdist>BstDst)
			{
				unsigned short x = StoredCoords[0];
				unsigned short y = StoredCoords[1];
				BestCoords[0] = x;
				BestCoords[1] = y;
				BestCoordsSet = 1;
				BstDst = newdist;
			}

		}



	}
}

void StartAI(unsigned short Depth)
{
	InitAIMove();
	DoAIMove(CursorX, CursorY, Depth);
}


// new ai
static unsigned short negativecount;
static unsigned short  positivecount;
static signed short smallest;
static signed short largest;
static unsigned short bestpos;
static unsigned short worstpos;

unsigned short getstats(unsigned short offset, unsigned short increment, unsigned short chosen)
{
	gridptr = PlayGrid+offset;

    negativecount = positivecount = r = 0;
    smallest = 255;
    largest = -255;

    /* grid_width/height are the same */
    while (r<grid_width)
	{
	    if (r!=chosen)
	    {
            /* value in grid */
            signed short val = gridptr[0];

            if (val!=0)
            {
                if (val<0)
                {
                    negativecount++;
                }
                else 
                {
                    positivecount++;
				}
				
                if (val<smallest)
                {
                    smallest = val;
                    worstpos = r;
                }
                if (val>largest)
                {
                    largest = val;
                    bestpos = r;
                }
            }
	    }
	    r++;
		gridptr+=increment;
	}

	if ((smallest==255) || (largest==-255))
	{
		return 0;
	}

	return 1;
}

//static signed int BstDst;
static signed short BstDstTheirScore;
static signed short BstDstNoMove;
static signed short BstDstNeg;
static signed short BstDstNegTheirScore;
//static int BestCoordsSet;
static unsigned	    short OurBestX;
static 	unsigned    short OurBestY;
static 	unsigned	short OurBestNoMoveX;
static 	unsigned	short OurBestNoMoveY;
static 	unsigned	short OurBestNegX;
static 	unsigned	short OurBestNegY;
 static unsigned short OurBestSet=0;
static 	unsigned	short OurBestNegSet=0;
static 	unsigned	short OurBestNoMoveSet=0;

static 	int checkmove;
int DoAIMoveNew(int xstart, int ystart, int Depth)
{

	if (Depth!=0)
	{
	  
		int x,y;
		int xinc, yinc;
		/* based on current player, choose where to scan grid */
		if (CurrentAIPlayer==0)
		{
			xinc=1;
			yinc=0;
			x = 0;
			y = ystart;
		}
		else
		{
			xinc=0;
			yinc=1;
			x = xstart;
			y = 0;
		}

		checkmove = 1;
	//	printf("----------------------------------\n");
//		printf("P1 Score: %d P2 Score %d\n\n", PlayerScores[0], PlayerScores[1]);
//		printf("Cursor: %d %d\n", xstart, ystart);
//		printf("We are: %d\n\n", CurrentAIPlayer);

		/* scan whole column or row */
		while ((x<grid_width) && (y<grid_height))
		{
			/* get tile */
			signed short Tile = GetTile(x,y);

			/* tile not taken */
			if (Tile!=0)
			{
                signed int dist = Tile;
				int movepossible;
				signed int tiledist;
				signed int scoredist;
				
			//	printf("======\n");
			 //   printf("Tile: %d\n\n",Tile);
              //  printf("Our Score after tile: %d\n", PlayerScores[StoredAIPlayer]+Tile);

			    if (CurrentAIPlayer==0)
			    {
			        /* column for next player */
			        movepossible = getstats(x, grid_width, y);
			    }
			    else
			    {
			        /* row for next player */
			        movepossible = getstats((y*grid_width), 1, x);
			    }

				if (movepossible)
				{
					signed short theirbest = (PlayerScores[StoredAIPlayer^1]+largest);
					scoredist = (PlayerScores[StoredAIPlayer]+Tile)-theirbest;
					tiledist = Tile-largest;

				//	printf("their worst: %d (%d) best: %d (%d) numpos: %d numneg: %d\n", smallest, worstpos, largest, bestpos, positivecount, negativecount);
		//			printf("Dist: %d\n", tiledist);
		//			printf("Dist Scores: %d\n", scoredist);

		//			printf("Their Best/Worst Score: %d/%d\n",PlayerScores[StoredAIPlayer^1]+largest, PlayerScores[StoredAIPlayer^1]+smallest);

		//			if (tiledist!=scoredist)
		//			{
		//				printf("adasddas");
		//			}
					

#if 0
				if (!OurBestSet)
                {

                    /* compare against tile when scores begin */
                    /* try and find tile that has the closest and largest distance */

                    /* find where our score would be better than theirs if we took the number */
                    if (Tile>=largest)
                    {
                        OurBestSet = 1;
                        OurBestX = x;
                        OurBestY = y;
                    }
                }
                else
                {
                    /* find where our score would be better than theirs if we took the number */
                    if (Tile>=largest)
                    {
                        OurBestX = x;
                        OurBestY = y;
                    }
                }
#endif
					// if lots of negative numbers, work out our score and their best
					// then we can pick a number and force them to take negative
					if (largest<0)
					{
						if (!OurBestNegSet)
						{
							OurBestNegSet = 1;
							OurBestNegX = x;
							OurBestNegY = y;
							BstDstNeg = scoredist;
							BstDstNegTheirScore = theirbest;
						}
						else 
						{
							if (scoredist>BstDstNeg)
							{
								OurBestNegX = x;
								OurBestNegY = y;
								BstDstNeg = scoredist;
							}
							else if (scoredist==BstDstNeg)
							{
								if ((theirbest)<BstDstTheirScore)
								{
									OurBestNegX = x;
									OurBestNegY = y;
									BstDstNeg = scoredist;
									BstDstNegTheirScore = theirbest;
								}

							//	printf("adadasd");

							}
						}
					}
					else
					{

						if (!OurBestSet)
						{

							/* compare against tile when scores begin */
							/* try and find tile that has the closest and largest distance */

							/* find where our score would be better than theirs if we took the number */
							OurBestSet = 1;
							OurBestX = x;
							OurBestY = y;
							BstDst = scoredist;//;Tile-largest;
							BstDstTheirScore = theirbest;
						}
						else
						{
							/* find where our score would be better than theirs if we took the number */
							if (scoredist>BstDst)
							{
								OurBestX = x;
								OurBestY = y;
								BstDst = scoredist;//Tile-largest;
								BstDstTheirScore = theirbest;
							}
							else if (scoredist==BstDst)
							{
								if ((theirbest)<BstDstTheirScore)
								{
									OurBestX = x;
									OurBestY = y;
									BstDst = scoredist;//Tile-largest;
									BstDstTheirScore = theirbest;
								}
							}

							// what about equal?

						}
					}
				}
				else
				{		
					tiledist = Tile;
					scoredist = (PlayerScores[StoredAIPlayer]+Tile)-PlayerScores[StoredAIPlayer^1];

					if (!OurBestNoMoveSet)
					{
						OurBestNoMoveX = x;
						OurBestNoMoveY = y;
						BstDstNoMove = scoredist;
						OurBestNoMoveSet = 1;
					}
					else
					{
						if (scoredist>BstDstNoMove)
						{
							OurBestNoMoveX = x;
							OurBestNoMoveY = y;
							BstDstNoMove = scoredist;
						}
						else if (scoredist==BstDstNoMove)
						{

						//	printf("adasdsdasdas");

						}
					}	
				}
			}
			/* increment x and y to move accross or down grid */
			x += xinc;
			y += yinc;
		}
	}


	{
		int BestestDist;
		int BestestX;
		int BestestY;
		int BestestSet = 0;

		if (OurBestSet)
		{
			BestestSet = 1;
			BestestX = OurBestX;
			BestestY = OurBestY;
			BestestDist = BstDst;
		}

		if (OurBestNegSet)
		{
			if ((!BestestSet) || (BstDstNeg>BestestDist))
			{
				BestestSet = 1;
				BestestX = OurBestNegX;
				BestestY = OurBestNegY;
				BestestDist = BstDstNeg;
			}
		}

		// if no move is set, and distance is +ve, it
		// means our score will be better and we can end game early
		if (OurBestNoMoveSet)
		{
			if (BstDstNoMove>0)
			{
				BestestX = OurBestNoMoveX;
				BestestY = OurBestNoMoveY;
				BestestDist = BstDstNoMove;
				BestestSet = 1;
			}
		}

		if (BestestSet)
		{
			BestCoords[0] = BestestX;
			BestCoords[1] = BestestY;
		
			return 0;
		}
	}


	return 1;
}



void DoAI()
{
	switch (CurrentDiff)
	{
		case DIFF_EASY:
		{
			if (PlayerId==0)
			{
				getstats(CursorY*grid_width, 1, CursorX);
				DestCursorX = bestpos;
				DestCursorY = CursorY;
			}
			else
			{
				getstats(CursorX, grid_width, CursorY);
				DestCursorX = CursorX;
				DestCursorY = bestpos;
			}
		}
		break;
	
		case DIFF_MEDIUM:
		{
			StartAI(2);
			DestCursorX = BestCoords[0];
			DestCursorY = BestCoords[1];
		}
		break;
		
		case DIFF_HARD:
		{
			StoredAIPlayer = CurrentAIPlayer = PlayerId;
			 OurBestSet=OurBestNegSet=OurBestNoMoveSet=0;
			DoAIMoveNew(CursorX, CursorY, 1);
		//	StartAI(4);
			DestCursorX = BestCoords[0];
			DestCursorY = BestCoords[1];
			 
		}
		break;
	}

	CurrentMode = MODE_MOVE_CURSOR;
	NextMode = MODE_TAKE_NUMBER;
}


static  unsigned char *DiffTxt[3] = {"EASY","MEDIUM","HARD"};
static  unsigned char *GameModeTxt[5] = {"NORMAL","TIMER 5","TIMER 10", "TIMER 15", "HIDDEN"};
static unsigned char *VSTxt[4] = {"P1 VS CPU","CPU VS P2", "P1 VS P2","CPU VS CPU"};
static unsigned char *StartTxt[4] = {"P1","P2","RANDOM","LAST WINNER"};
static unsigned char *SoundTxt[3] = {"MUSIC", "SFX","MUTE"};

static unsigned char *PauseMenu[2] = {"CONTINUE", "QUIT"};
static unsigned char *MainMenu[6] = {"DIFFICULTY:","MODE:","VS:","START MODE:","SOUND:", "PLAY"};


void set_text_inks (unsigned short i1,unsigned short i2,unsigned short i3,unsigned short i4) 
{
	cpc_SetInkGphStr (0, i1);
	cpc_SetInkGphStr (1, i2);
	cpc_SetInkGphStr (2, i3);
	cpc_SetInkGphStr (3, i4);
}

/* draw ui string */
void draw_string(unsigned char *StringList[], unsigned short Index, unsigned char *addr)
{
	/* set colour for drawing */
	set_text_inks(pen0,pen2, pen3, pen4);

	/* clear behind string */
	cpc_ClearRectangle((unsigned char *)addr, 11<<1, font_height, 0);
	
	/* now draw string */
	cpc_PrintGphStr2(StringList[Index], (unsigned char *)addr);
}


void draw_diff()
{
	draw_string(DiffTxt, Diff, SCR_ADDR_FROM_CHAR0(13, MAIN_MENU_Y+0));
}
	

void draw_gamemode()
{
	draw_string(GameModeTxt, GameMode, SCR_ADDR_FROM_CHAR0(13, MAIN_MENU_Y+1));
}


void draw_vs()
{
	draw_string(VSTxt, VS, SCR_ADDR_FROM_CHAR0(13, MAIN_MENU_Y+2));
}

void draw_startmode()
{
	draw_string(StartTxt, StartMode, SCR_ADDR_FROM_CHAR0(13, MAIN_MENU_Y+3));
}

void draw_soundmode()
{
	draw_string(SoundTxt, SoundMode, SCR_ADDR_FROM_CHAR0(13, MAIN_MENU_Y+4));
}



void draw_column(unsigned short Column)
{
	unsigned short y;
	for (y=0; y<grid_height; y++)
	{
		draw_grid_square(Column, y);
	}
}

void draw_row(unsigned short Row)
{
	unsigned short x;
	for (x=0; x<grid_width; x++)
	{
		draw_grid_square(x, Row);
	}
}

void draw_grid()
{
	/* draw grid */
	unsigned short y;
	draw_blank = 0;
	if (GameMode==GAMEMODE_HIDDEN)
	{
		draw_blank = 1;
	}
	for (y=0; y<grid_height; y++)
	{
		draw_row(y);
	}
	
	if (GameMode==GAMEMODE_HIDDEN)
	{
		draw_blank = 0;
		if (PlayerId==0)
		{
			draw_row(CursorY);
		}
		else 
		{
			draw_column(CursorX);
		}		
	}
	
}

void SetPlayerTypes()
{
	switch (VS)
	{
		case VS_PLAYERVSCPU:
		{
			PlayerType[1] = (unsigned short)PLAYER_CPU;
			PlayerType[0] = (unsigned short)PLAYER_HUMAN;
		}
		break;
		
		case VS_CPUVSPLAYER:
		{
			PlayerType[1] = (unsigned short)PLAYER_HUMAN;
			PlayerType[0] = (unsigned short)PLAYER_CPU;
		}
		break;

		case VS_PLAYERVSPLAYER:
		{
			PlayerType[1] = (unsigned short)PLAYER_HUMAN;
			PlayerType[0] = (unsigned short)PLAYER_HUMAN;
		}
		break;
		
		case VS_CPUVSCPU:
		{
			PlayerType[0] = PlayerType[1] = (unsigned short)PLAYER_CPU;
		}
		break;
	}
}		

void plot_menu_indicator(unsigned short x, unsigned short y, unsigned short index, unsigned char *ch)
{
	unsigned char *Addr = cpc_GetScrAddress(SCR_CHAR_TO_BYTEX0(x), SCR_CHAR_TO_LINEY0(index+y));
	cpc_PrintGphStr2(ch, Addr);
}

unsigned short debouncekey(unsigned short code)
{
	/* quicker to do this than ==0 */
	
	return (cpc_TestKeyFN(code) && (!cpc_TestPrevKeyFN(code)));
}


void wait_vsync()
{
	while (VsyncMark==0);
	VsyncMark = 0;
}

unsigned short debounce_up()
{
	return (debouncekey(0) || debouncekey(5));
}

unsigned short debounce_down()
{
	return (debouncekey(1) || debouncekey(6));
}

unsigned short debounce_fire()
{
	return (debouncekey(4) || debouncekey(9));
}

M_BEGIN_ISR(interrupt_handler)
{
	if (cpc_InVsync())
	{
		cpc_BackupKeys();
		cpc_ScanKeyboard();
	//	if (SoundMode!=SOUND_MUTE)
	//	{
			/* music or sfx */
			starkos_play_music();
	//	}
		/* ok, we are ready for code to continue that is waiting on us */

		VsyncMark = 1;
	
		
		/* NOTE: 
		1. This is effectively a random number based on time.
		2. We set the random seed when the game is started.
		3. We only ever start the game around vsync time.
		4. So there is no point incrementing the random seed at any other time
		because then we will end up with a seed which is not good.
		e.g. if we incremented it every interrupt we would end up with a seed which
		is a multiple of 6, and we would always get an even random number seed, the result,
		is we could not get random numbers in range 0-1 */
		
		
		RandomSeed++;
	}

}
M_END_ISR



void stop_starkos_music()
{
	cpc_DisableInterrupt();

	/* stop the sound */
	starkos_stop_sound();
}

void set_starkos_music(unsigned short issfx, unsigned char *current_sound)
{
	stop_starkos_music();	
		
	starkos_init_music(issfx, current_sound);

	cpc_InstallInterrupt(interrupt_handler);
	
	/* enable it */
	cpc_EnableInterrupt();
}



void play_sfx(unsigned char *current_sound)
{
	if (SoundMode==SOUND_SFX)
	{
		set_starkos_music(1,current_sound);
		while (!starkos_has_music_looped())
		{
			wait_vsync();
		}
	}
}	

#ifdef CASSETTE
void play_music(unsigned char *sound)
{
//	current_music = sound;
	if (SoundMode==SOUND_MUSIC)
	{
//		set_starkos_music(0, current_music);
		set_starkos_music(0, sound);
	}
}
#endif
#ifdef DISC
void play_music(int issfx, unsigned char *sound)
{
//	current_music = sound;
	if (SoundMode==SOUND_MUSIC)
	{
//		set_starkos_music(0, current_music);
		set_starkos_music(issfx, sound);
	}
}
#endif

void play_select_sfx()
{
//#ifdef DISC
	play_sfx(select_sfx);
//#endif
}

void play_move_sfx()
{
//#ifdef DISC
	play_sfx(move_sfx);
//#endif
}


static unsigned char numberstr[6];

void displaynumberRA(signed short number, unsigned short Addr)
{
	unsigned short len;
	/* convert to string and right align */
	len = cpc_ConvertNumberToAscii(numberstr, number, 0);
	cpc_PrintGphStr2(numberstr,Addr-(len<<1) );
}

void displayplayerscore(unsigned short Player)
{
#ifdef CASSETTE
	unsigned char *scraddrsrc;
	
	/* e.g. -  1,-100, 34 */
	cpc_ConvertNumberToAscii(numberstr, PlayerScores[Player] , 1);
		
	
	scraddrsrc = (Player==0) ? SCR_ADDR_FROM_COORD(69, 126) : SCR_ADDR_FROM_COORD(69, 172);
	
//	if (Player==0)
//	{
//		scraddrsrc = SCR_ADDR_FROM_COORD(69, 126);
//	}
//	else
//	{
//		scraddrsrc = SCR_ADDR_FROM_COORD(69, 172);
//	}

	/* oranges */
	set_text_inks(pen0,pen13,pen8,pen9);

	cpc_SetDoubleHeightChar();
	cpc_PrintGphStr2(numberstr, scraddrsrc);
	cpc_SetSingleHeightChar();
#endif

#ifdef DISC
	unsigned char *scraddr;
	unsigned short i;
	unsigned char *numberptr = numberstr;
	
	/* e.g. -  1,-100, 34 */
	unsigned short Length = cpc_ConvertNumberToAscii(numberstr, PlayerScores[Player] , 1);
		
	if (Player==0)
	{
#define scrx1 ((35*2)-1)
#define scry1 (16*8)-3
		scraddr = SCR_ADDR_FROM_COORD(scrx1, scry1);
	}
	else
	{
#define scrx2 ((35*2)-1)
#define scry2 (22*8)-4

		scraddr = SCR_ADDR_FROM_COORD(scrx2, scry2);
	}

	for (i=0; i<Length; i++)
	{
		unsigned char *TileGfx;
		unsigned char ch = *numberptr;
		++numberptr;
		if (ch==0x02d)
		{
			TileGfx = LargeScoreGfx;
		}
		else if (ch==0x02b)
		{
			TileGfx = LargeScoreGfx+(LARGE_SCORE_TILE_SIZE*1);
		}
		else if (ch==0x020)
		{
			TileGfx = LargeScoreGfx+(LARGE_SCORE_TILE_SIZE*2);
		}
		else
		{
			TileGfx = LargeScoreGfx+(LARGE_SCORE_TILE_SIZE*((ch-0x030)+3));
		}
		cpc_PutSp(TileGfx, LARGE_SCORE_TILE_HEIGHT, LARGE_SCORE_TILE_WIDTH_BYTES, scraddr);
		scraddr+=LARGE_SCORE_TILE_WIDTH_BYTES;
	}
#endif
}




void draw_logo()
{
		cpc_ClearRectangle(SCR_ADDR_FROM_COORD(playfield_org_x, playfield_org_y), playfield_width_bytes, playfield_height_lines,0);
		set_text_inks(pen0,pen2,pen3,pen4);

#ifdef CASSETTE
#define TITLE_X 6
#define TITLE_Y 3

		cpc_SetDoubleHeightChar();
		cpc_PrintGphStr2("BLUE ANGEL 69", SCR_ADDR_FROM_CHAR0(TITLE_X,TITLE_Y));
		cpc_SetSingleHeightChar();
#endif
#ifdef DISC	
			/* blue angel logo */
			cpc_PutSp(Logo, 55, 96>>1, SCR_ADDR_FROM_COORD(1,8));
		cpc_SetSingleHeightChar();
#endif				
}

#ifdef DISC

static unsigned char Loading[] = "LOADING.";
static unsigned char DiscMissing[] = "PLEASE INSERT GAME DISC";
static unsigned char PressAnyKey[] = "AND PRESS ANY KEY.";
static unsigned char BadDisc[] = "GAME DISC IS BAD.";
static unsigned char PressAnyKey2[] = "GAME HALTED.";

enum
{
	ERROR_CODE_NONE = 0,
	ERROR_CODE_DISC_MISSING,
	ERROR_CODE_BAD_DISC
};

void load_background()
{
	int errorCode;
	
	if (LoadedBackgroundIndex==BackgroundIndex)
		return;
		
	LoadedBackgroundIndex = BackgroundIndex;
	
	stop_starkos_music();
	
	/* restore firmware interrupt */
	cpc_RestoreFirmwareInterrupt();
		
	cpc_EnableInterrupt();
	
	/* turn off messages */
	cpc_EnableDTMessages(0);
	
	/* set filename (sizeof includes null terminator!) */
	Filename[sizeof(Filename)-2]=0x30+BackgroundIndex;

	do
	{
		int returnCode;
		errorCode = ERROR_CODE_NONE;
		draw_logo();
		cpc_PrintGphStr2(Loading, SCR_ADDR_FROM_CHAR0(9,14));	
		/* load background into buffer; not crunched */
		returnCode = cpc_LoadBinary(Filename,BackgroundBuffer); 
	
		/* amsdos is first to report? */
		if ((returnCode & AMSDOS_ERROR_CODE)!=0)
		{
			/* this could indicate disc is missing */
			if (returnCode == AMSDOS_ERROR_BAD_COMMAND) 
			{
				errorCode = ERROR_CODE_DISC_MISSING;
			}
			else if (returnCode == AMSDOS_ERROR_FILE_DOES_NOT_EXIST)
			{
				errorCode = ERROR_CODE_BAD_DISC;
			}
		}
		else if ((returnCode & DISC_CTRL_ERROR_CODE)!=0)
		{
			if ((returnCode & DISC_CTRL_ERROR_DRIVE_NOT_READY)!=0)
			{
				errorCode = ERROR_CODE_DISC_MISSING;
			}
		}

		if (errorCode==ERROR_CODE_DISC_MISSING)
		{
			/* display error message */
			cpc_PrintGphStr2(DiscMissing, SCR_ADDR_FROM_CHAR0(1,14));
			cpc_PrintGphStr2(PressAnyKey, SCR_ADDR_FROM_CHAR0(1,15));
		//	while(!cpc_AnyKeyPressed());
			cpc_WaitCharFirmware();
		}
		else if (errorCode == ERROR_CODE_BAD_DISC)
		{
			cpc_PrintGphStr2(BadDisc, SCR_ADDR_FROM_CHAR0(4,14));
			cpc_PrintGphStr2(PressAnyKey2, SCR_ADDR_FROM_CHAR0(6,15));
		//	while(!cpc_AnyKeyPressed());
			//cpc_WaitCharFirmware();
			/* hang */
			while(1==1) {}
		}
	}
	while (errorCode!=ERROR_CODE_NONE);
}
#endif


unsigned short continuemenu(unsigned short itemcount, unsigned short x,unsigned short y, unsigned short itemsel)
{
	unsigned short selected = 0;
	unsigned short item = itemsel;
	
	plot_menu_indicator(x,y,item,"#");
	while (selected==0)
	{
		wait_vsync();
				
		if (debounce_fire())
		{	
			play_select_sfx();
				
			selected = 1;
		}
		else
		if (debounce_up())
		{
			if (item!=0)
			{
				play_move_sfx();
				plot_menu_indicator(x,y,item," ");
				item--;
				plot_menu_indicator(x,y,item,"#");
			}
		}
		/* down */
		else if (debounce_down())
		{
			if (item!=(itemcount-1))
			{
				play_move_sfx();
				plot_menu_indicator(x,y,item," ");
				item++;
				plot_menu_indicator(x,y,item,"#");
			}
		}		
	
	}
	return item;
}

/* draw menu inks */
void initmenu(unsigned short itemcount, unsigned char *items[], unsigned short x, unsigned short y)
{
	unsigned short i;
	unsigned short xbyte = SCR_CHAR_TO_BYTEX0((x+1));
	unsigned short yline = SCR_CHAR_TO_LINEY0(y);
	
	set_text_inks(pen0,pen13, pen9,pen14);

	for (i=0; i<itemcount; i++)
	{
		unsigned char *Addr = cpc_GetScrAddress(xbyte, yline);
		cpc_PrintGphStr2(items[i], Addr);
		yline+=font_height;
	}
}

void try_take_number()
{
	if (GetTile(CursorX, CursorY)!=0)
	{
		/* take it */
		CurrentMode = MODE_TAKE_NUMBER;
	}
}

void DrawTimer()
{
	/* draw timer */
	{
		#define MAX_TIMER_BAR_LENGTH 44
		unsigned short BarLength = (Timer*MAX_TIMER_BAR_LENGTH)/TimerReset;
		cpc_DrawColouredBar0(SCR_ADDR_FROM_COORD(53,155),BarLength, 4, PEN_BYTE0(pen9));

	}
}
#ifdef CASSETTE
static const unsigned char *GoTxt = "<<";
static const unsigned char *StopTxt = "--";
#endif

#ifdef CASSETTE
static const unsigned char *WinTxt[2] = {"()","!\""};
static const unsigned char *LoseTxt[2] = {"%&","?@"};
#endif

void draw_stop_go()
{
#if DISC
	unsigned char *gfxaddr1;
	unsigned char *gfxaddr2;
	if (PlayerId==0)
	{
		gfxaddr1 = GO_GFX;
		gfxaddr2 = STOP_GFX;
	}
	else
	{
		gfxaddr1 = STOP_GFX;
		gfxaddr2 = GO_GFX;
	}
	cpc_PutSpTr(gfxaddr1, STOP_GO_GFX_HEIGHT, STOP_GO_GFX_WIDTH_BYTES, SCR_ADDR_FROM_COORD(stopgox1, stopgoy1));
	cpc_PutSpTr(gfxaddr2, STOP_GO_GFX_HEIGHT, STOP_GO_GFX_WIDTH_BYTES, SCR_ADDR_FROM_COORD(stopgox2, stopgoy2));
#endif
#ifdef CASSETTE
	unsigned char *stopaddr;
	unsigned char *goaddr;
	
	if (PlayerId==0)
	{
		goaddr = SCR_ADDR_FROM_COORD(64,126);
		stopaddr = SCR_ADDR_FROM_COORD(64,172);
	}
	else
	{
		goaddr = SCR_ADDR_FROM_COORD(64,172);
		stopaddr = SCR_ADDR_FROM_COORD(64,126);
	}
	cpc_SetDoubleHeightChar();
	set_text_inks(pen0,pen13,pen8,pen9);
	cpc_PrintGphStr2(StopTxt, stopaddr);
	set_text_inks(pen0,pen15,pen11,pen14);
	cpc_PrintGphStr2(GoTxt, goaddr);
	cpc_SetSingleHeightChar();
#endif

}

	
#define winlosescrx1 (32*2)-1
#define winlosescry1 (16*8)-4
#define winlosescrx2  (32*2)-1
#define winlosescry2  (22*8)-6


#define lastscrx1 57
#define lastscry1 122
#define lastscrx2 57
#define lastscry2 168

void blank_winlose()
{
	PlayerId = 0;
	cpc_ClearRectangle(SCR_ADDR_FROM_COORD(winlosescrx1, winlosescry1), WIN_LOSE_TILE_WIDTH_BYTES,WIN_LOSE_TILE_HEIGHT, 0);
	cpc_ClearRectangle(SCR_ADDR_FROM_COORD(winlosescrx2, winlosescry2), WIN_LOSE_TILE_WIDTH_BYTES,WIN_LOSE_TILE_HEIGHT, 0);

	PlayerScores[0] = PlayerScores[1] = 0;
	displayplayerscore(0);
	displayplayerscore(1);
	TimerReset = Timer = 2;
	DrawTimer();
	draw_stop_go();
#define CLEAR_TILE_ADDR (Tiles+(22*tile_size))
	tilescraddr =SCR_ADDR_FROM_COORD(lastscrx1, lastscry1);
	draw_trans_tile(CLEAR_TILE_ADDR);
	tilescraddr =SCR_ADDR_FROM_COORD(lastscrx2, lastscry2);
	draw_trans_tile(CLEAR_TILE_ADDR);
}

static unsigned short KeyTable[]={
	/* Q */			
	0, 0x4808,
	/* A */
	1, 0x4820,
	/* O */			
	2, 0x4404,
	/* P */
	3, 0x4308,
	/* space */
	4, 0x4580,
	/* joy up */			
	5, 0x4901,
	/* joy down */
	6, 0x4902,
	/* joy left */			
	7, 0x4904,
	/* joy right */
	8, 0x4908,
	/* fire */
	9, 0x4910,
	/* H pause*/
	10, 0x4510,
	/* ESC */
	11, 0x04804
};

void draw_win()
{
#ifdef DISC
	cpc_PutSp(WinGfx[WinnerAnimFrame], WIN_LOSE_TILE_HEIGHT, WIN_LOSE_TILE_WIDTH_BYTES, WinLoseScrAddr);
#endif
#ifdef CASSETTE
	set_text_inks(pen0,pen13,pen12, pen14);
	cpc_PrintGphStr2(WinTxt[WinnerAnimFrame], WinLoseScrAddr);
#endif
	
}

void draw_lose()
{
#ifdef DISC
	cpc_PutSp(LoseGfx[WinnerAnimFrame], WIN_LOSE_TILE_HEIGHT, WIN_LOSE_TILE_WIDTH_BYTES, WinLoseScrAddr);
#endif
#ifdef CASSETTE
	set_text_inks(pen0,pen15,pen8,pen5);
	cpc_PrintGphStr2(LoseTxt[WinnerAnimFrame], WinLoseScrAddr);
#endif
}

void main()
{
	/* initialise default random seed */
	RandomSeed = cpc_GetRRegister();
	
	starkos_reloc_song(main_music, main_music);
	
//#ifdef DISC
	starkos_reloc_song(select_sfx, select_sfx);
	starkos_reloc_song(move_sfx, move_sfx);
#if DISC
	starkos_reloc_song(game_music, game_music);
	starkos_reloc_song(win_music, win_music);
	starkos_reloc_song(lost_music, lost_music);
#endif
	
#ifdef DISC
	WinGfx[0] = WIN_HEART_GFX;
	WinGfx[1] = WIN_HEART_GFX2;
	LoseGfx[0] = WIN_SKULL_GFX;
	LoseGfx[1] = WIN_SKULL_GFX2;
#endif
#ifdef DISC
	cpc_DisableInterrupt();
	/* store firmware interrupt vector */	
	cpc_StoreFirmwareInterrupt();
	
	cpc_EnableInterrupt();
	
	cpc_SetBorder(0);
	cpc_WaitFirmwareColourChange();
#endif

#ifdef CASSETTE
	cpc_SetMode(0);
#endif
	cpc_AssignMultipleKey(12, KeyTable);
#ifdef CASSETTE				
	// start music playing first, we don't want to restart it unless we need to
	play_music(main_music);
#endif		
	
	/* Q */			
//	cpc_AssignKey(0, 0x4808);
	/* A */
//	cpc_AssignKey(1, 0x4820);
	/* O */			
//	cpc_AssignKey(2, 0x4404);
	/* P */
//	cpc_AssignKey(3, 0x4308);
	/* space */
//	cpc_AssignKey(4, 0x4580);

	/* joy up */			
//	cpc_AssignKey(5, 0x4901);
	/* joy down */
//	cpc_AssignKey(6, 0x4902);
	/* joy left */			
//	cpc_AssignKey(7, 0x4904);
	/* joy right */
//	cpc_AssignKey(8, 0x4908);
	/* fire */
//	cpc_AssignKey(9, 0x4910);
	
	/* H pause*/
//	cpc_AssignKey(10, 0x4510);
	
	/* ESC */
//	cpc_AssignKey(11, 0x04804);

	/* inks, mode and crtc are set by loader */
	

	while (1==1)
	{
		switch (scr)
		{
			case SCR_FRONTEND:
			{
				unsigned short donemenu;
				unsigned short item;

				draw_logo();

				cpc_PrintGphStr2("CODE:  KEVIN THACKER",SCR_ADDR_FROM_CHAR0(1,20));
				cpc_PrintGphStr2("GFX:   MARKUS HOHMANN",SCR_ADDR_FROM_CHAR0(1,21));
				cpc_PrintGphStr2("MUSIC: ROALD STRAUSS",SCR_ADDR_FROM_CHAR0(1,22));
#ifdef DISC				
				play_music(0,main_music);
#endif		
				initmenu(6,MainMenu,MAIN_MENU_X,MAIN_MENU_Y);
	
				/*DrawTimer();*/

				draw_diff();
				draw_gamemode();
				draw_vs();
				draw_startmode();
				draw_soundmode();
				
				blank_winlose();
				
				item = 0;
				donemenu = 0;
				while (donemenu==0)
				{
				
					item = continuemenu(6,MAIN_MENU_X,MAIN_MENU_Y,item);
				
					switch (item)
					{
						case OP_DIFF:
						{
							Diff++;
							if (Diff==DIFF_MAX)
								Diff=DIFF_EASY;
							
							draw_diff();
						}
						break;
						
						case OP_GAMEMODE:
						{
							GameMode++;
							if (GameMode==GAMEMODE_MAX)
								GameMode=GAMEMODE_NORMAL;
							
							draw_gamemode();
						}
						break;

						case OP_VS:
						{
							VS++;
							if (VS==VS_MAX)
								VS=VS_PLAYERVSCPU;
							
							draw_vs();
						}
						break;
			
						case OP_STARTMODE:
						{
							StartMode++;
							if (StartMode==START_MAX)
								StartMode = START_PLAYER1;
								
							draw_startmode();
						}
						break;
						
						case OP_SOUND:
						{
							SoundMode++;
							if (SoundMode==SOUND_MAX)
								SoundMode = SOUND_MUSIC;
								
							/* if music, start sound again */
							if (SoundMode==SOUND_MUSIC)
							{
								set_starkos_music(0,main_music);	//current_music);
							}
							else
							{
								/* sfx or mute */
								/* stop sound */
								set_starkos_music(0,0);
							}

								
							draw_soundmode();
						}
						break;
								
						case OP_PLAY:
						{
							donemenu = 1;
						}
						break;
					}					
				}
					
				// reset games won
				GamesWon[0] = GamesWon[1] = 0;
						
				WinnerId = -1;
				
				scr = SCR_GAME;
			}
			break;
			
			case SCR_GAME:
			{
				/* game */
				/*unsigned short x,y;*/
				unsigned short game_active = 1;
				CurrentMode = MODE_START;
				BackgroundIndex = 0;
				SetPlayerTypes();
				
				CurrentDiff = Diff;
				RoundCount = 0;			
													
				while (game_active)
				{
					wait_vsync();

					if (ShowWinner)
					{
						/* performing winner delay? */
						if (WinnerDelay!=0)
						{
							/* update delay */
							WinnerDelay--;
						}
						else
						{
							/* done delay, now update frame */
#ifdef DISC						
							WinLoseScrAddr = SCR_ADDR_FROM_COORD(winlosescrx1, winlosescry1);			
							(WinnerId==0)? draw_win() : draw_lose();
							WinLoseScrAddr = SCR_ADDR_FROM_COORD(winlosescrx2, winlosescry2);
							(WinnerId==1)? draw_win() : draw_lose();
#endif							
#ifdef CASSETTE
							cpc_SetDoubleHeightChar();
							WinLoseScrAddr = SCR_ADDR_FROM_COORD(64,125);			
							(WinnerId==0)? draw_win() : draw_lose();
							WinLoseScrAddr = SCR_ADDR_FROM_COORD(64,171);
							(WinnerId==1)? draw_win() : draw_lose();
							cpc_SetSingleHeightChar();
#endif
							WinnerAnimFrame^=1;	
							WinnerDelay = 10;
						}
					}
					
					switch (CurrentMode)
					{
						case MODE_START:
						{
							/* ensure we don't get a random seed of 0 */
							{
								/* initialise random seed */
								int Seed = RandomSeed;
								if (Seed==0)
								{
									/* some default seed */
									Seed=0x0aa;
								}
									
								cpc_SetRandSeed(Seed);
							}
#ifdef DISC
							/* load initial background */
							load_background();
#endif
#ifdef CASSETTE
							copy_background();
#endif


#ifdef DISC				
							{
								if (SoundMode!=SOUND_MUSIC)
								{
									set_starkos_music(0,0);
								}
								else
								{
									play_music(0,game_music);
								}
							}
#endif				

							/* clear screen */

							cpc_ClearRectangle(
										SCR_ADDR_FROM_COORD(playfield_org_x, playfield_org_y), 
										playfield_width_bytes, 
										playfield_height_lines,
										0
										);
							TimerReset = 5*50;
							switch (GameMode)
							{
								case GAMEMODE_TIMER5:
								{
									TimerReset = 5*50;
								}
								break;
								
								case GAMEMODE_TIMER10:
								{
									TimerReset = 10*50;
								}
								break;
								
								case GAMEMODE_TIMER15:
								{
									TimerReset = 15*50;
								}
								break;
							}

							Timer = TimerReset;

							switch (StartMode)
							{
								case START_PLAYER1:
								{
									PlayerId = StartPlayer = 0;
								}
								break;
								
								case START_PLAYER2:
								{
									PlayerId = StartPlayer = 1;
								}
								break;
								
								case START_RANDOM:
								{
									PlayerId = StartPlayer = cpc_RandomInRange(2);
								}
								break;
								
								case START_LASTWINNER:
								{
									if (WinnerId == -1)
									{
										PlayerId = StartPlayer = 0;
									}
									else
									{
										PlayerId = StartPlayer = WinnerId;
									}
								}
								break;
							}
							
							WinnerId = -1;
							
							/* reset scores */
							PlayerScores[0] = PlayerScores[1] = 0;
				
							{
								unsigned short sc;
								unsigned short c;
								//signed short *pg;
								unsigned short choices[grid_size];
								unsigned short choicesremain;
								unsigned short y;
								
								unsigned short Cursor = cpc_Random()&(grid_size-1);
								CursorX = Cursor%grid_width;
								CursorY = Cursor/grid_height;
								OldCursorX = CursorX;
								OldCursorY = CursorY;
								
								/* init choices array; list of all possible choices as
								offset values into PlayGrid */
								for (sc = 0; sc<grid_size; sc++)
								{
									choices[sc] = sc; 
								}
								
								/* exclude this choice */
								choices[Cursor] = 255;
								choicesremain = grid_size-1;
													
								for (y=0; y<grid_size; y++)
								{
									PlayGrid[y] = 0;
								}
											
								
								/* setup grid */
								
								/* one by one we make a choice from each of the possible offsets
								that are not used */
								/* and the number of possibilities decreases each time we make a choice */
								for (sc = 0; sc<sizeof(Scores)/sizeof(unsigned short); sc++)
								{
									/* make a choice out of the remaining elements in the array */
									unsigned short scchoice = cpc_RandomInRange(choicesremain);
									
									c = 0;
									
									/* find the actual offset that is available */
									while (c<grid_size)
									{
										/* -1 indicates this offset has already been chosen */
										if (choices[c]!=255)
										{
											if (scchoice==0)
												break;
												
											scchoice--;
										}
										c++;
									}
									
									{
										signed short scval = Scores[sc];
										
										/* found position and store */
										PlayGrid[c] = scval;
										
										/* say this choice has been made */
										choices[c] = 255;
										/* update number of choices */
										choicesremain--;
									}					
								}	
							}
								
							draw_grid();
							
							draw_cursor();	
							
							displayplayerscore(0);
							displayplayerscore(1);
							DrawTimer();
							draw_stop_go();
							CurrentMode = MODE_PLAYER_MOVE;						
						}
						break;
						
						case MODE_PAUSE:
						{
							unsigned short donemenu;
							unsigned short item = 0;
														
							draw_logo();

							//set_text_inks(pen0,pen2,pen3,pen4);

							cpc_PrintGphStr2("PAUSED",SCR_ADDR_FROM_CHAR0(9,PAUSE_MENU_Y));

							initmenu(2,PauseMenu,PAUSE_MENU_X,PAUSE_MENU_Y+3);
	
							donemenu = 0;
							while (donemenu==0)
							{
								item = continuemenu(2,PAUSE_MENU_X,PAUSE_MENU_Y+3,item);
							
								switch (item)
								{
									case PAUSE_OP_CONTINUE:
									{
										draw_grid();
										draw_cursor();
										
										CurrentMode = MODE_PLAYER_MOVE;
										donemenu = 1;
									}
									break;
									
									case PAUSE_OP_QUIT:
									{
										CurrentMode = MODE_BACK_TO_FRONT_END;
										donemenu = 1;
									}
									break;
								}
							}
						}
						break;
										
					
						case MODE_DELAY:
						{
							Delay--;
							if (Delay==0)
							{
								CurrentMode = DelayNextMode;
							}	
						}
						break;
						
						case MODE_TAKE_NUMBER:
						{
							unsigned short gameOver;
							signed short Tile = RemoveTile(CursorX, CursorY);
							unsigned char *StopGoAddr;
							
							/*LastTook[PlayerId] = Tile; */
	
							/* show tile taken */

							play_select_sfx();
							
							tilescraddr = (PlayerId==0) ? SCR_ADDR_FROM_COORD(lastscrx1, lastscry1) : SCR_ADDR_FROM_COORD(lastscrx2, lastscry2);
							
//							if (PlayerId==0)
//							{
//								tilescraddr = SCR_ADDR_FROM_COORD(lastscrx1, lastscry1);
//							
//							}
//							else 
//							{
//								tilescraddr = SCR_ADDR_FROM_COORD(lastscrx2, lastscry2);
//							}
							
							{	
								unsigned char *tileaddr = get_tile_addr(Tile);
								draw_trans_tile(tileaddr);
							}
							
							tilescraddr = (PlayerId==0) ? SCR_ADDR_FROM_COORD(lastscrx2, lastscry2) : SCR_ADDR_FROM_COORD(lastscrx1, lastscry1);
	
				//			if (PlayerId==0)
				//			{
				//				tilescraddr =SCR_ADDR_FROM_COORD(lastscrx2, lastscry2);
				//			}
				//			else
				//			{
				//				tilescraddr =SCR_ADDR_FROM_COORD(lastscrx1, lastscry1);
				//			}
							
							draw_trans_tile(CLEAR_TILE_ADDR);

							/* update score */
							PlayerScores[PlayerId]+=(signed short)Tile;
							
							displayplayerscore(PlayerId);
														
							if (PlayerId==0)
							{
								/* check if no moves can be made for player 2 */
								gameOver = checkEmpty(CursorX, grid_width, grid_height);
							}
							else
							{
								/* check if no moves can be made for player 1 */
								gameOver = checkEmpty((CursorY*grid_width), 1, grid_width);
							}
						
							/* reset timer */
							Timer = TimerReset;
							/* redraw timer */
							DrawTimer();

	
							if (gameOver!=0)
							{
								/* redraw grid if on hidden mode to hide tiles we are not allowed to see */							
								if (GameMode==GAMEMODE_HIDDEN)
								{
									if (PlayerId==0)
									{
										draw_blank = 1;
										draw_row(CursorY);
										draw_blank = 0;
										draw_column(CursorX);
									}
									else
									{
										draw_blank = 1;
										draw_column(CursorX);
										draw_blank = 0;
										draw_row(CursorY);
									}
								}							
								/* swap player */
								PlayerId ^= 1;
								
															
								CurrentMode = MODE_PLAYER_MOVE;
								
								DrawTimer();
								
								draw_cursor(); 
								draw_stop_go();
											
							}
							else
							{
								tilescraddr =SCR_ADDR_FROM_COORD(lastscrx1, lastscry1);
								draw_trans_tile(CLEAR_TILE_ADDR);
								tilescraddr =SCR_ADDR_FROM_COORD(lastscrx2, lastscry2);
								draw_trans_tile(CLEAR_TILE_ADDR);								
								
								draw_grid_square(CursorX, CursorY);
								
								CurrentMode = MODE_GRID_DONE;
							}
						}
						break;
	
						case MODE_GRID_REMOVE_TILES:
						{
							unsigned int Continue = 1;
							while (Continue)
							{
								/* get tile and remove from grid */
								signed short Tile = RemoveTile(CursorX, CursorY);
								
								/* was it blank? */
								if (Tile!=0)
								{
									play_select_sfx();
									/* no draw background */
									draw_blank = 0;
									draw_grid_square(CursorX, CursorY);

									/* delay a bit */
									DelayNextMode = MODE_GRID_REMOVE_TILES;
									Delay = 8;
									CurrentMode = MODE_DELAY;
									break;	
								}
								
								/* increment position */
								CursorX++;
								if (CursorX==grid_width)
								{
									CursorX = 0;
									CursorY++;
									if (CursorY==grid_height)
									{
										Continue = 0;
										CurrentMode = WinnerMode;
									}
								}
							}
						}
						break;

						case MODE_GRID_PLAYER_WIN:
						{	
							/* reveal picture one line at a time */
							cpc_CopySpriteRectangleToScreen((unsigned char *)cpc_GetScrAddress(background_org_x, background_org_y+showline), 
									1,
									background_width_bytes,
									BackgroundBuffer,
									0,
									showline,
									background_width_bytes);						
				
							/* increment line to show */
							showline++;

							if (showline!=playfield_height_lines)
							{
								/* delay before showing next line */
								CurrentMode = MODE_DELAY;
								DelayNextMode = MODE_GRID_PLAYER_WIN;
								Delay = 1;
							}
							else
							{
								/* shown all, get ready to show picture */
								DelayNextMode = MODE_GRID_DONE_DONE;
								
								/* setup delay to show picture; this is part of delay, go into next mode and delay some more */
								Delay = 3*50;
								CurrentMode = MODE_DELAY;
								
								/* increment background */
								BackgroundIndex++;
								if (BackgroundIndex>=NumBackgrounds)
									BackgroundIndex = 0;							
							}
						}
						break;
						
						case MODE_GRID_DONE_DONE:
						{
#ifdef DISC
							if (
								((VS==VS_PLAYERVSCPU) || (VS==VS_CPUVSPLAYER))
								)
							{
								if (SoundMode==SOUND_MUSIC)
								{
									if (!starkos_has_music_looped())
									{
										break;
									}
								}
								CurrentMode = MODE_DELAY;
							
								DelayNextMode = MODE_STATS;

								Delay = 2 * 50;
							}
							else
#endif							
							{
								/* cpu vs cpu, or player vs player */
									
								/* wait for a bit; this delay is cpu vs cpu, player vs player */
								Delay = 2 * 50;
								CurrentMode = MODE_DELAY;
								
								DelayNextMode = (VS==VS_CPUVSCPU) ? MODE_BACK_TO_FRONT_END : MODE_STATS;
							}							
						}
						break;
#if 0						
						case MODE_GRID_SHOW_WINNER:
						{
#ifdef DISC
							WinnerAnimRepeatCount--;
							if (WinnerAnimRepeatCount==0)
							{
								/* show winner icon and wait for sound to play */
								CurrentMode = WinnerMode;
								
							}
							else
							{
								WinnerAnimFrame^=1;	
								CurrentMode = MODE_DELAY;							
								Delay = 10;
								DelayNextMode = MODE_GRID_SHOW_WINNER;
							}

							cpc_PutSp(WinGfx[WinnerAnimFrame], WIN_LOSE_TILE_HEIGHT, WIN_LOSE_TILE_WIDTH_BYTES, WinnerScrAddr);
							cpc_PutSp(LoseGfx[WinnerAnimFrame],WIN_LOSE_TILE_HEIGHT, WIN_LOSE_TILE_WIDTH_BYTES, LoserScrAddr);
							
#endif
#ifdef CASSETTE
							cpc_SetDoubleHeightChar();
							
							/* heart colours */
							set_text_inks(pen0,pen13,pen14, pen12);
							cpc_PrintGphStr2(WinTxt, WinnerScrAddr);
							
							/* skull colours */
							set_text_inks(pen0,pen15,pen8,pen5);
							cpc_PrintGphStr2(LoseTxt, LoserScrAddr);
							cpc_SetSingleHeightChar();
							CurrentMode = WinnerMode;
#endif
						}
						break;
#endif
						case MODE_GRID_DONE:
						{
							/* who won? */
							
							/* testing */
							//PlayerScores[0] = 200;
							//PlayerScores[1] = 200;
							
							/* player 1 won? */
							if (PlayerScores[0]>PlayerScores[1])
							{
								WinnerId = 0;
							}
							/* player 2 won? */
							else if (PlayerScores[1]>PlayerScores[0])
							{
								WinnerId = 1;
							}
#if 0							
							/* draw */
							else
							{
								/* if playing against cpu and there was a draw, we declare cpu the winner */
								/* is this correct? */
								if (VS==VS_PLAYERVSCPU)
								{
									WinnerId = 1;
								}
								else if (VS==VS_CPUVSPLAYER)
								{
									WinnerId = 0;
								}
								else
								{
									/* otherwise it is the player who started */
									WinnerId = StartPlayer;
								}
							}
#endif

							/* remove tiles */
							CursorX = CursorY = 0;
													
							ShowWinner = 1;
							WinnerDelay = WinnerAnimFrame = 0;
							if (WinnerId!=-1)
							{
								GamesWon[WinnerId]++;

								/* player vs cpu? */
								if (
									((VS==VS_PLAYERVSCPU) || (VS==VS_CPUVSPLAYER))
								)
								{
									if (PlayerType[WinnerId]==PLAYER_HUMAN)
									{
										/* player won? */
										/* remove tiles and reveal picture */
										
										CurrentMode = MODE_GRID_REMOVE_TILES;
										WinnerMode = MODE_GRID_PLAYER_WIN;

										/* if we win 3 times we then go to a higher difficulty */
										RoundCount++;
										if (RoundCount==3)
										{
											RoundCount=0;
											CurrentDiff++;
											if (CurrentDiff==DIFF_MAX)
											{
												CurrentDiff = DIFF_HARD;
											}
										}
											
										/* reveal picture without grid */
										showline = 0;
#ifdef DISC								
										play_music(1,win_music);
								//		current_music = win_music;	
								//		set_starkos_music(0,current_music);
#endif
									}
									else
									{
										/* player didn't win. leave tiles, do not reveal picture */
										CurrentMode = MODE_GRID_DONE_DONE;
									
#ifdef DISC						
										play_music(1,lost_music);
//										current_music = lost_music;
//										set_starkos_music(0,current_music);
#endif
									}
								}	
								else
								{
									/* cpu vs cpu or player vs player */
									/* do not remove tiles and do not reveal picture */
									CurrentMode = MODE_GRID_DONE_DONE;
								}
							}
							else
							{
								/* draw */
								CurrentMode = MODE_GRID_DONE_DONE;
								Delay = 3*50;
							}
						}
						break;
						
						case MODE_STATS:
						{
							unsigned short donemenu;
							unsigned short item = 0;
	
							/* do not show winner anymore */
							ShowWinner = 0;
#ifdef DISC							
							play_music(0,main_music);
#endif							
					//		if (SoundMode==SOUND_MUSIC)
					//		{
//								current_music = main_music;	
//								set_starkos_music(0,current_music);
					//		}
					//		else
					//		{
					//			set_starkos_music(0,0);
					//		}
							
							draw_logo();
								
							//set_text_inks(pen0,pen2,pen3,pen4);
							
							if (VS!=VS_PLAYERVSPLAYER)
							{
								cpc_PrintGphStr2(MainMenu[0], SCR_ADDR_FROM_CHAR0(5,STAT_MENU_Y-2));
							}							
							cpc_PrintGphStr2("SCORE  PL1  PL2",SCR_ADDR_FROM_CHAR0(6,STAT_MENU_Y));
							cpc_PrintGphStr2("---------------",SCR_ADDR_FROM_CHAR0(6,STAT_MENU_Y+1));
							cpc_PrintGphStr2("LAST",SCR_ADDR_FROM_CHAR0(6, STAT_MENU_Y+2));
							cpc_PrintGphStr2("GAMES",SCR_ADDR_FROM_CHAR0(6, STAT_MENU_Y+3));
							
							set_text_inks(pen0,pen6,pen7,pen5);

							displaynumberRA(PlayerScores[0], SCR_ADDR_FROM_CHAR0(16,STAT_MENU_Y+2));
							displaynumberRA(PlayerScores[1], SCR_ADDR_FROM_CHAR0(21,STAT_MENU_Y+2));

							displaynumberRA(GamesWon[0], SCR_ADDR_FROM_CHAR0(16,STAT_MENU_Y+3));
							displaynumberRA(GamesWon[1], SCR_ADDR_FROM_CHAR0(21,STAT_MENU_Y+3));

							if (VS!=VS_PLAYERVSPLAYER)
							{
								cpc_PrintGphStr2(DiffTxt[CurrentDiff], SCR_ADDR_FROM_CHAR0(17, STAT_MENU_Y-2));
							}							
							initmenu(2,PauseMenu,STAT_MENU_X,STAT_MENU_Y+6);
	
							donemenu = 0;
							while (donemenu==0)
							{
								
								item = continuemenu(2,STAT_MENU_X,STAT_MENU_Y+6,item);
							
								switch (item)
								{
									case STAT_OP_CONTINUE:
									{
										CurrentMode = MODE_START;
										donemenu = 1;
									}
									break;
									
									case STAT_OP_QUIT:
									{
										
										CurrentMode = MODE_BACK_TO_FRONT_END;
										donemenu = 1;
									}
									break;
								}
							}

							/* blank win lose */
							blank_winlose();
							
							/* redraw timer? */
							
						}
						break;
						
						case MODE_BACK_TO_FRONT_END:
						{							
							/* do not show winner anymore */
							ShowWinner = 0;
							game_active = 0;
						}
						break;
#if 0												
						case MODE_SWAP_PLAYER:
						{

							/* redraw grid if on hidden mode to hide tiles we are not allowed to see */							
							if (GameMode==GAMEMODE_HIDDEN)
							{
								if (PlayerId==0)
								{
									draw_blank = 1;
									draw_row(CursorY);
									draw_blank = 0;
									draw_column(CursorX);
								}
								else
								{
									draw_blank = 1;
									draw_column(CursorX);
									draw_blank= 0;
									draw_row(CursorY);
								}
							}							
							/* swap player */
							PlayerId ^= 1;
							
														
							CurrentMode = MODE_PLAYER_MOVE;
							
							DrawTimer();

						}
						break;
#endif
					
						case MODE_MOVE_CURSOR:
						{
							OldCursorX = CursorX;
							OldCursorY = CursorY;
							if ((CursorX==DestCursorX) && (CursorY==DestCursorY))
							{
								/* go to next state */
								CurrentMode = NextMode;
							}
							else
							{
								play_move_sfx();
	
								if (CursorX!=DestCursorX)
								{
									if (DestCursorX<CursorX)
									{
										CursorX--;
									}
									else  if (DestCursorX>CursorX)
									{
										CursorX++;
									}
								}
								
								if (CursorY!=DestCursorY)
								{
									if (DestCursorY<CursorY)
									{
										CursorY--;
									}
									else if (DestCursorY>CursorY)
									{
										CursorY++;
									}
								}
								
								/* erase in old position, draw in new */
								draw_cursor();
								
								if (PlayerType[PlayerId]==PLAYER_CPU)
								{
									/* wait 10 frames before next move */
									Delay = 5;
									CurrentMode = MODE_DELAY;
									DelayNextMode = MODE_MOVE_CURSOR;
								}
							}							
						}
						break;
						
						case MODE_PLAYER_MOVE:
						{
							
							/* check for pause */
							if (cpc_TestKeyFN(10))
							{
								CurrentMode = MODE_PAUSE;
								break;
							}

							/* esc to quit */
							if (cpc_TestKeyFN(11))
							{
								CurrentMode = MODE_BACK_TO_FRONT_END;
								break;
							}							
							
							if (
							/* timer mode */
							((GameMode>=GAMEMODE_TIMER5) && (GameMode<=GAMEMODE_TIMER15)) && 
							/* current player is human */
							(PlayerType[PlayerId]==PLAYER_HUMAN))
							{
								Timer--;
								if (Timer==0)
								{
									Timer = TimerReset;
									
									if (PlayerId==0)
									{
										getstats(CursorY*grid_width, 1, CursorX);
										DestCursorX = worstpos;
										DestCursorY = CursorY;
									}
									else
									{
										getstats(CursorX, grid_width, CursorY);
										DestCursorX = CursorX;
										DestCursorY = worstpos;
									}
#if 0
									if (PlayerId==0)
									{
										DestCursorX = findBestOrWorst(CursorY*grid_width, 1, grid_width,1);
										DestCursorY = CursorY;

									}
									else
									{
										DestCursorY = findBestOrWorst(CursorX, 1, grid_height,1);
										DestCursorX = CursorX;
									}
#endif
									CurrentMode = 	MODE_MOVE_CURSOR;
									NextMode = MODE_TAKE_NUMBER;
									
									break;
								}
								
								DrawTimer();
							}
					
																							
							if (PlayerId==0)
							{											
								if (PlayerType[0]==PLAYER_HUMAN)
								{
									/* player 1 only move left/right */
																							
									/* move left? */
									
									if (debouncekey(2) || debouncekey(7))
									{
										if (CursorX!=0)
										{
											DestCursorX = CursorX-1;
											DestCursorY = CursorY;
											NextMode = MODE_PLAYER_MOVE;
											CurrentMode = MODE_MOVE_CURSOR;
										}
									}
									/* move right? */
									else if (debouncekey(3) || debouncekey(8))
									{
										if (CursorX!=(grid_width-1))
										{
											DestCursorX = CursorX+1;
											DestCursorY = CursorY;
											NextMode = MODE_PLAYER_MOVE;
											CurrentMode = MODE_MOVE_CURSOR;
										}
									}
									/* select? */
									else if (debounce_fire())
									{
										try_take_number();
									}
									
											
									
								}
								else
								{
									DoAI();
								}
							}
							else 
							{
								/* player 2 only move up/down */
								
								
								if (PlayerType[1]==PLAYER_HUMAN)
								{
									/* move up? */
									if (debounce_up())
									{
					
										if (CursorY!=0)
										{
											DestCursorY = CursorY-1;
											DestCursorX = CursorX;
											NextMode = MODE_PLAYER_MOVE;
											CurrentMode = MODE_MOVE_CURSOR;
	
										}
									}
									/* move down? */
									else if (debounce_down())
									{
										if (CursorY!=(grid_height-1))
										{
											DestCursorY = CursorY+1;
											DestCursorX = CursorX;
											NextMode = MODE_PLAYER_MOVE;
											CurrentMode = MODE_MOVE_CURSOR;
										}
									}
									/* select? */
									else if (debounce_fire())
									{
										try_take_number();
									}					
								}
								else
								{
									DoAI();
								}
							}
						}
						break;
					}			
					
				}
				
				scr = SCR_FRONTEND;
			}
			break;
		}
	}
}
