
num_roll_deltas equ (roll_deltas_end-roll_deltas)/2

roll_start:
call make_scr_table

;; start by drawing line 0
ld hl,1
ld (y_dst),hl

do_roll:
ld b,&f5
in a,(c)
rra
jr nc,do_roll

call draw_roll

ld hl,(y_dst)
inc hl
ld (y_dst),hl
call cp_200
jp m,do_roll
ret

get_scr_addr:
push de
add hl,hl
ld de,scr_addr_table
add hl,de
ld a,(hl)
inc hl
ld h,(hl)
ld l,a
pop de
ret

cp_200:
or a
ld bc,200
sbc hl,bc
add hl,bc
ret


draw_roll:

;; copy actual gfx line (not part of barrel roll)

ld hl,(y_dst)				;; scanline position to draw barrel to
dec hl						;; scanline position to draw actual line to
call get_scr_addr
;; HL = screen address
push hl
ld e,l						;; DE = same as dest
ld d,h
call copy_line				;; copy line
pop de			

;; DE = screen address of line drawn
;; HL = line (for barrel)

;; now draw barrel
ld ix,roll_deltas			;; table of delta values
ld b,num_roll_deltas		;; number of lines to draw
draw_roll_loop:
push bc

;; DE go to next scanline for drawing
ex de,hl
call scr_next_line
ex de,hl

;; DE = scanline for drawing
push de

ld hl,(y_dst)
ld c,(ix+0)
ld b,(ix+1)
add hl,bc
inc ix
inc ix
;; IX = next delta value

;; HL = adjusted scanline

;; are we over end of source image?
;; yes we can skip drawing this line
call cp_200
jp p,skip_draw

;; lookup screen address for adjusted scanline
call get_scr_addr
;; now copy this line 
call copy_line

skip_draw:
pop de

pop bc
djnz draw_roll_loop
ret

copy_line:
ld a,h
and &3f
or &40
ld h,a

ld a,d
and &3f
or &c0
ld d,a					;; de = dest
						;; hl = source
ld bc,80				;; length of line
ldir
ret


y_dst:
defw 0

y_src:
defw 0

;; if height is 8
;; then we need to start 8 ahead
;; from then we show highest lines at top
;; and smaller at bottom
;; near middle we see more lines or lines repeated or stretched
roll_deltas:
defw 1+4+3+2+1+1+2+3+3+4+4+num_roll_deltas
defw 1+4+3+2+1+1+2+3+3+4+num_roll_deltas
defw 1+4+3+2+1+1+2+3+3+num_roll_deltas
defw 1+4+3+2+1+1+2+3+num_roll_deltas
defw 1+4+3+2+1+1+2+num_roll_deltas			
defw 1+4+3+2+1+1+num_roll_deltas
defw 1+4+3+2+1+num_roll_deltas
defw 1+4+3+2+num_roll_deltas
defw 1+4+3+num_roll_deltas
defw 1+4+3+num_roll_deltas
defw 1+4+num_roll_deltas
defw 1+4+num_roll_deltas
defw 1+num_roll_deltas
;; lowest
roll_deltas_end:

;; 4
;; 4
;; 3
;; 3
;; 2
;; 1
;; 1
;; 2
;; 3
;; 3
;; 4
;; 4


;; generate table of screen addresses
;; one address per scanline. The address
;; is for the first byte of each line.
make_scr_table:
ld ix,scr_addr_table		;; address to store table
ld hl,&c000					;; start address of first scanline
ld b,200					;; number of scanlines on screen
mst1:
push bc
ld (ix+0),l
ld (ix+1),h
inc ix
inc ix
call scr_next_line
pop bc
djnz mst1
ret

scr_next_line:
ld a,h
add a,8
ld h,a
ret nc
ld a,l
add a,&50
ld l,a
ld a,h
adc a,&c0
ld h,a
ret


scr_addr_table:
defs 200*2

