;; Topo Soft loader
;; 
;; I also call this "Spanish" because of the multi colours in the 
;; border which is typical to most Spanish tape loaders.
;;
;; This loader uses standard Spectrum ROM timings.

;; IX = start address
;; DE = length
;; 
;; bit 7 of R register must be 0.
;; Set like this:
;; xor a
;; ld r,a
;; doesn't corrupt alternative register set
topoload:
di      
ex      af,af'
exx     
push    af
push    de
push    bc
ld      d,&10				;; Gate-Array pen index
							;; The colour of this pen is changed when tape is read
ld      b,&7f				;; Gate-Array port address upper 8 bits
exx     
							;; Assumption: PPI port A is setup for writing
							
ld      bc,&f40e			;; select AY register 14 (for reading keyboard)
out     (c),c
ld      bc,&f6d0			;; "AY write register" and enable tape motor
out     (c),c
ld      c,&10				;; "AY inactive" and enable tape motor
out     (c),c
ld      bc,&f792			;; set PPI port A to read
out     (c),c
ld      bc,&f658			;; tape motor enable and select keyboard row 8
							;; (z, caps lock, a, tab, q, esc, 2, 1)
							;; we are only interested in ESC
out     (c),c

call    l81a3
ld      a,(l8084)
scf     
call    l80f8

;; set border colour to black
ld      bc,&7f10
out     (c),c
ld      a,&54
out     (c),a


ex      af,af'
exx     
pop     bc
pop     de
pop     af
exx     
ex      af,af'
ld      bc,&f782				;; set PPI port A for output
out     (c),c
ld      bc,&f610				;; tape motor on, "AY inactive"
out     (c),c
push    af
call    l81a7
ld      a,&ff
ld      (l8084),a
pop     af
ret     

l8084:
defb &ff

l80e8: ld      a,r
and     &08
or      &44
ld      c,&11
ld      b,&7f
out     (c),c
out     (c),a
jr      l80e8

l80f8: 
inc     d
ex      af,af'
dec     d
ld      a,&f5			;; PPI port B
in      a,(&00)			;; read port (tape read etc)
and     &80				;; isolate tape read data
exx     
ld      e,&4c
exx     
ld      c,a
cp      a
l8107: 
nop     
l8108: 
call    l817b
jr      nc,l8107

ld      hl,&415
l8110: 
djnz    l8110
dec     hl
ld      a,h
or      l
jr      nz,l8110

call    l8177
jr      nc,l8107

l811c: 
ld      b,&9c
call    l8177
jr      nc,l8107
ld      a,&b9
cp      b
jr      nc,l8108
inc     h
jr      nz,l811c

l812b: 
ld      b,&c9
call    l817b
jr      nc,l8107
ld      a,b
cp      &d1
jr      nc,l812b

l8137: 
call    l817b
ret     nc

exx     
ld      e,&4a
exx     
ld      h,&00
ld      b,&b0
jr      l815d

l8145: 
ex      af,af'
jr      nz,l814d
;; L = data byte read from cassette
;; store to RAM
ld      (ix+&00),l
jr      l8157

l814d: 
rr      c
xor     l
ret     nz

ld      a,c
rla     
ld      c,a
inc     de
jr      l8159

l8157: 
inc     ix
l8159: 
dec     de
ex      af,af'
ld      b,&b2
l815d: 
ld      l,&01
l815f: 
call    l8177
ret     nc

ld      a,&c3
cp      b
rl      l
ld      b,&b0
jr      nc,l815f

ld      a,h
xor     l
ld      h,a
ld      a,d
or      e
jr      nz,l8145
ld      a,h
cp      &01
ret     

l8177: 
call    l817b
ret     nc

l817b: 
ld      a,&16
l817d: 
dec     a
jr      nz,l817d

and     a
l8181: 
inc     b
ret     z

;; read keyboard
ld      a,&f4			;; PPI port A 
in      a,(00h)			;; read port and read keyboard through AY register 14
and     &04				;; isolate state of bit 2 (ESC key)
						;; bit will be 0 if key is pressed
xor     &04				;; has key been pressed?
ret     nz				;; quit

ld      a,&f5			;; PPI port B
in      a,(&00)			;; read port (tape read etc)
xor     c
and     &80				;; isolate tape read
jr      z,l8181
ld      a,c				;; effectively toggle bit 7 of C
cpl     				;; which is the tape read state we want to see next
ld      c,a
exx     
ld      a,r				;; R is incremented for each opcode fetched
						;; and is almost like a very crude random number generator
						;; here it is used to choose a colour for the pen to be
						;; note only the lower 7-bits are incremented.
						;; Bit 7 is fixed by LD R,A. Here bit 7 must be 0!
						;; Gate-Array pen colour will also only be set when bit 7 is set to 0
						;; and bit 6 is set to 1. If bit 6 is set to 0, then pen index will be chosen
						;; which will cause no effect
ld      e,a
out     (c),d			;; select Gate Array pen
out     (c),e			;; write colour to pen
exx     
scf     
 ret     

l81a3: 
ld      a,&10
jr      l81a9

l81a7: 
ld      a,&ef
l81a9: 
push    bc
ld      b,&f6
in      c,(c)
inc     b
and     &10
ld      a,&08
jr      z,l81b6
inc     a
l81b6: 
out     (c),a
pop     bc
ret     
